import 'dart:convert';
import 'package:test/test.dart';
import 'package:http/http.dart' as http;

import 'package:pip_services3_commons/pip_services3_commons.dart';
import 'package:service_drones_dart/src/data/version1/data.dart';
import 'package:service_drones_dart/src/logic/logic.dart';
import 'package:service_drones_dart/src/persistence/persistence.dart';
import 'package:service_drones_dart/src/services/version1/service.dart';

final DRONE1 = DroneV1(
    id: '1',
    type: DroneTypeV1.MiniDrone,
    ownerId: '0001',
    serial: '123',
    range: 100,
    registered: DateTime.now(),
    base: LocationV1(lat: 50, long: 70));

final DRONE2 = DroneV1(
    id: '2',
    type: DroneTypeV1.MiniDrone,
    ownerId: '0002',
    serial: '456',
    range: 200,
    registered: DateTime.now(),
    base: LocationV1(lat: 30, long: 40));

var httpConfig = ConfigParams.fromTuples([
  'connection.protocol',
  'http',
  'connection.host',
  'localhost',
  'connection.port',
  3000
]);

void main() {
  group('DronesCommandableServiceV1', () {
    late DronesMemoryPersistence persistence;
    late DronesController controller;
    late DronesCommandableServiceV1 service;
    late http.Client rest;
    late String url;

    setUp(() async {
      url = 'http://localhost:3000';
      rest = http.Client();

      persistence = DronesMemoryPersistence();
      persistence.configure(ConfigParams());

      controller = DronesController();
      controller.configure(ConfigParams());

      service = DronesCommandableServiceV1();
      service.configure(httpConfig);

      var references = References.fromTuples([
        Descriptor('eic-stopfires-services-drones', 'persistence', 'memory',
            'default', '1.0'),
        persistence,
        Descriptor('eic-stopfires-services-drones', 'controller', 'default',
            'default', '1.0'),
        controller,
        Descriptor('eic-stopfires-services-drones', 'service', 'http',
            'default', '1.0'),
        service
      ]);

      controller.setReferences(references);
      service.setReferences(references);

      await persistence.open(null);
      await service.open(null);
    });

    tearDown(() async {
      await service.close(null);
      await persistence.close(null);
    });

    test('CRUD Operations', () async {
      DroneV1 drone1;

      // Create the first drone
      var resp = await rest.post(Uri.parse(url + '/v1/drones/create_drone'),
          headers: {'Content-Type': 'application/json'},
          body: jsonEncode({'drone': DRONE1}));
      var drone = DroneV1();
      drone.fromJson(jsonDecode(resp.body));
      expect(drone, isNotNull);
      expect(DRONE1.type, drone.type);
      expect(DRONE1.ownerId, drone.ownerId);
      expect(DRONE1.serial, drone.serial);
      expect(DRONE1.range, drone.range);
      expect(DRONE1.registered, drone.registered);
      expect(DRONE1.base, isNotNull);

      // Create the second drone
      resp = await rest.post(Uri.parse(url + '/v1/drones/create_drone'),
          headers: {'Content-Type': 'application/json'},
          body: jsonEncode({'drone': DRONE2}));
      drone = DroneV1();
      drone.fromJson(jsonDecode(resp.body));
      expect(drone, isNotNull);
      expect(DRONE2.type, drone.type);
      expect(DRONE2.ownerId, drone.ownerId);
      expect(DRONE2.serial, drone.serial);
      expect(DRONE2.range, drone.range);
      expect(DRONE2.registered, drone.registered);
      expect(DRONE2.base, isNotNull);

      // Get all drones
      resp = await rest.post(Uri.parse(url + '/v1/drones/get_drones'),
          headers: {'Content-Type': 'application/json'},
          body:
              jsonEncode({'filter': FilterParams(), 'paging': PagingParams()}));
      var page = DataPage<DroneV1>.fromJson(json.decode(resp.body), (item) {
        var drone = DroneV1();
        drone.fromJson(item);
        return drone;
      });
      expect(page, isNotNull);
      expect(page.data.length, 2);

      drone1 = page.data[0];

      // Update the drone
      drone1.label = 'ABC';

      resp = await rest.post(Uri.parse(url + '/v1/drones/update_drone'),
          headers: {'Content-Type': 'application/json'},
          body: json.encode({'drone': drone1}));
      drone = DroneV1();
      drone.fromJson(json.decode(resp.body));
      expect(drone, isNotNull);
      expect(drone1.id, drone.id);
      expect('ABC', drone.label);

      // Get drone by udi
      resp = await rest.post(Uri.parse(url + '/v1/drones/get_drone_by_serial'),
          headers: {'Content-Type': 'application/json'},
          body: json.encode({'serial': drone1.serial}));
      drone = DroneV1();
      drone.fromJson(json.decode(resp.body));
      expect(drone, isNotNull);
      expect(drone1.id, drone.id);

      // Delete the drone
      resp = await rest.post(Uri.parse(url + '/v1/drones/delete_drone_by_id'),
          headers: {'Content-Type': 'application/json'},
          body: json.encode({'drone_id': drone1.id}));
      drone = DroneV1();
      drone.fromJson(json.decode(resp.body));
      expect(drone, isNotNull);
      expect(drone1.id, drone.id);

      // Try to get deleted drone
      resp = await rest.post(Uri.parse(url + '/v1/drones/get_drone_by_id'),
          headers: {'Content-Type': 'application/json'},
          body: json.encode({'drone_id': drone1.id}));
      expect(resp.body, isEmpty);
    });
  });
}
